import { InMemoryCache } from "apollo-cache-inmemory";
import { ApolloClient, ApolloError, ApolloQueryResult, MutationOptions, QueryOptions } from "apollo-client";
import { ApolloLink, execute, FetchResult } from "apollo-link";
import { onError } from "apollo-link-error";
import { createUploadLink } from "apollo-upload-client";

// TODO: IE support
// const parseHeaders = (rawHeaders: string) => {
// 	const headers: Record<string, string> = {};
// 	// Replace instances of \r\n and \n followed by at least one space or horizontal tab with a space
// 	// https://tools.ietf.org/html/rfc7230#section-3.2
// 	const preProcessedHeaders = rawHeaders.replace(/\r?\n[\t ]+/g, " ");
// 	preProcessedHeaders.split(/\r?\n/).forEach(line => {
// 		const parts = line.split(":");
// 		const key = parts.shift()!.trim();
// 		if (key) {
// 			const value = parts.join(":").trim();
// 			headers[key] = value;
// 		}
// 	});
// 	return headers;
// };

// const customFetch: (context: any) => (input: RequestInfo, init?: RequestInit) => Promise<Response> = context => (
// 	url,
// 	options = {}
// ) =>
// 	new Promise((resolve, reject) => {
// 		const xhr = new XMLHttpRequest();
// 		const { headers, onProgress } = context;

// 		xhr.onload = () => {
// 			const opts = {
// 				status: xhr.status,
// 				statusText: xhr.statusText,
// 				headers: parseHeaders(xhr.getAllResponseHeaders() || "")
// 			};

// 			// @ts-ignore
// 			opts.url = "responseURL" in xhr ? xhr.responseURL : opts.headers["X-Request-URL"];
// 			// @ts-ignore
// 			const body = "response" in xhr ? xhr.response : xhr.responseText;
// 			resolve(new Response(body, opts));
// 		};

// 		xhr.onerror = () => {
// 			xhr.abort();
// 			reject(new TypeError("Network request failed"));
// 		};

// 		xhr.ontimeout = () => {
// 			reject(new TypeError("Network request failed"));
// 		};

// 		xhr.onabort = () => {
// 			reject(new TypeError("Network request failed"));
// 		};

// 		// @ts-ignore
// 		xhr.open(options.method, url, true);

// 		// @ts-ignore
// 		Object.keys(options.headers).forEach(key => {
// 			// @ts-ignore
// 			xhr.setRequestHeader(key, options.headers[key]);
// 		});

// 		if (headers)
// 			Object.keys(headers).forEach(key => {
// 				xhr.setRequestHeader(key, headers[key]);
// 			});

// 		if (xhr.upload) xhr.upload.onprogress = onProgress;

// 		xhr.send(options.body);
// 	});

export const client = new ApolloClient({
	link: ApolloLink.from([
		onError((/*{ graphQLErrors, networkError }*/) => {
			// if (graphQLErrors)
			// 	graphQLErrors.map(({ message, locations, path }) =>
			// 		console.log(`[GraphQL error]: Message: ${message}, Location: ${locations}, Path: ${path}`)
			// 	);
			// if (networkError) console.log(`[Network error]: ${networkError}`);
		}),
		new ApolloLink(operation => {
			operation.setContext((context: any = {}) => {
				const headers = { ...context.headers };

				if (context.token) headers["x-token"] = context.token;

				return {
					...context,
					headers
				};
			});

			return execute(
				createUploadLink({
					uri: "https://open.fing.edu.uy/api-preview/v1/graphql",
					credentials: "same-origin",
					headers: {
						Accept: "application/json"
					}
				}),
				operation
			);
		})
	]),
	cache: new InMemoryCache(),
	defaultOptions: {
		query: {
			fetchPolicy: "no-cache"
		}
	}
});

export const query = <TData, TVariables, TRequest extends BaseQueryRequest>(
	request: TRequest,
	options: QueryOptions<TVariables>
): Promise<SuccessfulQueryResponse<TData> | FailedQueryResponse> => {
	return client
		.query<TData, TVariables>(options)
		.then(response => ({
			success: true as true,
			rawResponse: response
		}))
		.catch((reason: ApolloError) => ({
			success: false as false,
			rawResponse: reason
		}));
};

export const mutate = <TData, TVariables, TRequest extends BaseMutationRequest>(
	request: TRequest,
	options: MutationOptions<any, TVariables>
): Promise<SuccessfulMutationResponse<TData> | FailedMutationResponse> => {
	return client
		.mutate(options)
		.then(response => ({
			success: true as true,
			rawResponse: response
		}))
		.catch((reason: ApolloError) => ({
			success: false as false,
			rawResponse: reason
		}));
};

export type BaseRequest = {
	authToken?: string;
};
export type BaseQueryRequest = BaseRequest;
export type BaseMutationRequest = BaseRequest;

export type SuccessfulQueryResponse<TData> = {
	success: true;
	rawResponse: ApolloQueryResult<TData>;
};
export type FailedQueryResponse = {
	success: false;
	rawResponse: ApolloError;
};

export type SuccessfulMutationResponse<TData> = {
	success: true;
	rawResponse: FetchResult<TData>;
};
export type FailedMutationResponse = {
	success: false;
	rawResponse: ApolloError;
};
