import gql from "graphql-tag";
import * as Errors from "../errors";
import * as Factories from "../factories";
import * as Models from "../models";
import { BaseQueryRequest, query } from "./Base";

// region fetchCourses
export type FetchCoursesRequest = BaseQueryRequest;
export type FetchCoursesData = {
	courses: Array<{
		id: number;
		code: string;
		name?: string;
		eva?: string;
		iconURL?: string;
	}>;
};
export type FetchCoursesVariables = {};
export type SuccessfulFetchCoursesResponse = {
	success: true;
	courses: Models.Course[];
};
export type FailedFetchCoursesResponse = {
	success: false;
	error: Errors.GenericError;
};
export type FetchCoursesResponse = SuccessfulFetchCoursesResponse | FailedFetchCoursesResponse;
export const fetchCourses = async (request: FetchCoursesRequest = {}): Promise<FetchCoursesResponse> => {
	const apiResponse = await query<FetchCoursesData, FetchCoursesVariables, FetchCoursesRequest>(request, {
		query: gql`
			{
				courses {
					id
					code
					name
					eva
					iconURL
				}
			}
		`,
		variables: {}
	});

	if (apiResponse.success) {
		const { data } = apiResponse.rawResponse;

		if (data && Array.isArray(data.courses))
			return {
				success: true,
				courses: data.courses.map(Factories.courseFromJSON)
			};
	}

	return {
		success: false,
		error: Errors.genericError("Hubo un error al intentar obtener la lista de cursos.")
	};
};
// endregion

// region fetchCourseByCode
export type FetchCourseByCodeRequest = BaseQueryRequest & { code: string };
export type FetchCourseByCodeData = {
	courseByCode?: Nullable<{
		id: number;
		code: string;
		name?: Nullable<string>;
		eva?: Nullable<string>;
		semester?: Nullable<number>;
		year?: Nullable<number>;
		iconURL?: Nullable<string>;

		classLists?: Nullable<
			Array<{
				id: number;
				name?: Nullable<string>;

				classes?: Nullable<
					Array<{
						id: number;
						title?: Nullable<string>;
						number?: Nullable<number>;

						videos: Nullable<
							Array<{
								id: number;
								name?: Nullable<string>;
								position?: Nullable<number>;

								qualities: Nullable<
									Array<{
										id: number;
										height?: Nullable<number>;
										width?: Nullable<number>;

										formats: Nullable<
											Array<{
												id: number;
												name?: Nullable<string>;
												url?: Nullable<string>;
											}>
										>;
									}>
								>;
							}>
						>;
					}>
				>;
			}>
		>;
	}>;
};
export type FetchCourseByCodeVariables = { code: string };
export type SuccessfulFetchCourseByCodeResponse = {
	success: true;
	course: Models.Course;
};
export type FailedFetchCourseByCodeResponse = {
	success: false;
	error: Errors.GenericError;
};
export type FetchCourseByCodeResponse = SuccessfulFetchCourseByCodeResponse | FailedFetchCourseByCodeResponse;
export const fetchCourseByCode = async (request: FetchCourseByCodeRequest): Promise<FetchCourseByCodeResponse> => {
	const { code } = request;

	const apiResponse = await query<FetchCourseByCodeData, FetchCourseByCodeVariables, FetchCourseByCodeRequest>(
		request,
		{
			query: gql`
				query courseByCode($code: String!) {
					courseByCode(code: $code) {
						id
						code
						name
						eva
						semester
						year
						iconURL

						classLists {
							id
							name

							classes {
								id
								title
								number

								videos {
									id
									name
									position

									qualities {
										id
										height
										width

										formats {
											id
											name
											url
										}
									}
								}
							}
						}
					}
				}
			`,
			variables: { code }
		}
	);

	if (apiResponse.success) {
		const { data } = apiResponse.rawResponse;

		if (data && data.courseByCode) {
			return {
				success: true,
				course: Factories.courseFromJSON(data.courseByCode)
			};
		}
	}

	return {
		success: false,
		error: Errors.genericError("Hubo un error al intentar obtener el curso.")
	};
};
// endregion
