import gql from "graphql-tag";
import * as Errors from "../errors";
import * as Factories from "../factories";
import * as Models from "../models";
import { BaseQueryRequest, query } from "./Base";

// region fetchCourseClassByClassNo
export type FetchCourseClassByClassNoRequest = BaseQueryRequest & {
	courseCode: string;
	classNo: number;
};
export type FetchCourseClassByClassNoData = {
	courseClassByClassNo: Nullable<{
		id: number;
		number?: Nullable<number>;
		title?: Nullable<string>;

		videos: Nullable<
			Array<{
				id: number;
				name?: Nullable<string>;
				position?: Nullable<number>;

				qualities: Nullable<
					Array<{
						id: number;
						height?: Nullable<number>;
						width?: Nullable<number>;

						formats: Nullable<
							Array<{
								id: number;
								name?: Nullable<string>;
								url?: Nullable<string>;
							}>
						>;
					}>
				>;
			}>
		>;
	}>;
};
export type FetchCourseClassByClassNoVariables = {
	courseCode: string;
	classNo: number;
};
export type SuccessfulFetchCourseClassByClassNoResponse = {
	success: true;
	courseClass: Models.CourseClass;
};
export type FailedFetchCourseClassByClassNoResponse = {
	success: false;
	error: Errors.GenericError;
};
export type FetchCourseClassByClassNoResponse =
	| SuccessfulFetchCourseClassByClassNoResponse
	| FailedFetchCourseClassByClassNoResponse;

export const fetchCourseClassByClassNo = async (
	request: FetchCourseClassByClassNoRequest
): Promise<FetchCourseClassByClassNoResponse> => {
	const { courseCode, classNo } = request;

	const response = await query<
		FetchCourseClassByClassNoData,
		FetchCourseClassByClassNoVariables,
		FetchCourseClassByClassNoRequest
	>(request, {
		query: gql`
			query courseClassByClassNo($courseCode: String!, $classNo: Int!) {
				courseClassByClassNo(courseCode: $courseCode, classNo: $classNo) {
					id
					number
					title

					videos {
						id
						name
						position

						qualities {
							id
							height
							width

							formats {
								id
								name
								url
							}
						}
					}
				}
			}
		`,
		variables: { courseCode, classNo }
	});

	if (response.success) {
		const { data } = response.rawResponse;

		if (data && data.courseClassByClassNo)
			return {
				success: true,
				courseClass: Factories.courseClassFromJSON(data.courseClassByClassNo)
			};
	}

	return {
		success: false,
		error: Errors.genericError("Hubo un error al intentar obtener la clase.")
	};
};
// endregion

// region fetchLatestCourseClasses
export type FetchLatestCourseClassesRequest = {};
export type FetchLatestCourseClassesData = {
	latestCourseClasses: Nullable<
		Array<{
			id: number;
			title?: Nullable<string>;
			number?: Nullable<number>;

			createdAt?: Nullable<string>;

			courseClassList?: Nullable<{
				id: number;
				course?: Nullable<{
					id: number;
					code?: Nullable<string>;
					name?: Nullable<string>;
					iconURL?: Nullable<string>;
				}>;
			}>;
		}>
	>;
};
export type FetchLatestCourseClassesVariables = {};
export type SuccessfulFetchLatestCourseClassesResponse = {
	success: true;
	courseClasses: Models.CourseClass[];
};
export type FailedFetchLatestCourseClassesResponse = {
	success: false;
	error: Errors.GenericError;
};
export type FetchLatestCourseClassesResponse =
	| SuccessfulFetchLatestCourseClassesResponse
	| FailedFetchLatestCourseClassesResponse;

export const fetchLatestCourseClasses = async (
	request: FetchLatestCourseClassesRequest = {}
): Promise<FetchLatestCourseClassesResponse> => {
	const response = await query<
		FetchLatestCourseClassesData,
		FetchLatestCourseClassesVariables,
		FetchLatestCourseClassesRequest
	>(request, {
		query: gql`
			{
				latestCourseClasses {
					id
					title
					number

					createdAt

					courseClassList {
						id
						course {
							id
							code
							name
							iconURL
						}
					}
				}
			}
		`
	});

	if (response.success) {
		const { data } = response.rawResponse;

		if (data && Array.isArray(data.latestCourseClasses))
			return {
				success: true,
				courseClasses: data.latestCourseClasses.map(Factories.courseClassFromJSON)
			};
	}

	return {
		success: false,
		error: Errors.genericError("Hubo un error al intentar obtener la clase.")
	};
};
// endregion
