import { useCallback, useContext, useEffect, useRef, useState } from "react";
import * as API from "../api";
import { RootStoreContext } from "../context/RootStoreContext";
import * as Errors from "../errors";
import * as Models from "../models";

export const useFetchCourses = ({ autoFetch }: UseFetchCoursesInput): UseFetchCoursesOutput => {
	const rootStore = useContext(RootStoreContext);
	const { courseStore, userStore } = rootStore;
	const requestRef = useRef<API.FetchCoursesRequest>();
	const [state, setState] = useState<FetchCoursesState>({ isLoading: false });

	const fetch = useCallback<() => Promise<FetchCoursesResponse>>(async () => {
		setState({ isLoading: true });

		const request: API.FetchCoursesRequest = { authToken: userStore.authToken };

		requestRef.current = request;
		const response = await API.fetchCourses(request);

		const newState: FetchCoursesResponse = response.success
			? {
					isLoading: false,
					success: true,
					courses: response.courses
			  }
			: {
					isLoading: false,
					success: false,
					error: response.error
			  };

		if (requestRef.current !== request) return newState;

		if (response.success) {
			response.courses.forEach(course => courseStore.saveCourse(course));
			courseStore.courseList = response.courses;
		}

		setState(newState);
		return newState;
	}, [courseStore]);

	useEffect(() => {
		setState({ isLoading: false });
		if (autoFetch && courseStore.courseList.length === 0) fetch();

		return () => (requestRef.current = undefined);
	}, [autoFetch, fetch, courseStore]);

	return [state, fetch];
};

export type InitialFetchCoursesState = {
	isLoading: false;
	success?: undefined;
};
export type LoadingFetchCoursesState = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulFetchCoursesState = {
	isLoading: false;
	success: true;
	courses: Models.Course[];
};
export type FailedFetchCoursesState = {
	isLoading: false;
	success: false;
	error: Errors.GenericError;
};
export type FetchCoursesResponse = SuccessfulFetchCoursesState | FailedFetchCoursesState;
export type FetchCoursesState = InitialFetchCoursesState | LoadingFetchCoursesState | FetchCoursesResponse;

export type UseFetchCoursesInput = {
	autoFetch: boolean;
};
export type UseFetchCoursesOutput = [FetchCoursesState, () => Promise<FetchCoursesResponse>];
