import { useCallback, useContext, useEffect, useRef, useState } from "react";
import * as API from "../api";
import { RootStoreContext } from "../context/RootStoreContext";
import * as Errors from "../errors";
import * as Models from "../models";

export const useFetchFAQs = ({ autoFetch }: UseFetchFAQsInput): UseFetchFAQsOutput => {
	const rootStore = useContext(RootStoreContext);
	const { faqStore, userStore } = rootStore;
	const requestRef = useRef<API.FetchFAQsRequest>();
	const [state, setState] = useState<FetchFAQsState>({ isLoading: false });

	const fetch = useCallback<() => Promise<FetchFAQsResponse>>(async () => {
		setState({ isLoading: true });

		const request: API.FetchFAQsRequest = { authToken: userStore.authToken };

		requestRef.current = request;
		const response = await API.fetchFAQs(request);

		const newState: FetchFAQsResponse = response.success
			? {
					isLoading: false,
					success: true,
					faqs: response.faqs
			  }
			: {
					isLoading: false,
					success: false,
					error: response.error
			  };

		if (requestRef.current !== request) return newState;

		if (response.success) {
			response.faqs.forEach(faq => faqStore.saveFAQ(faq));
			faqStore.faqList = response.faqs;
		}

		setState(newState);
		return newState;
	}, [faqStore]);

	useEffect(() => {
		setState({ isLoading: false });
		if (autoFetch && faqStore.faqList.length === 0) fetch();

		return () => (requestRef.current = undefined);
	}, [autoFetch, fetch, faqStore]);

	return [state, fetch];
};

export type InitialFetchFAQsState = {
	isLoading: false;
	success?: undefined;
};
export type LoadingFetchFAQsState = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulFetchFAQsState = {
	isLoading: false;
	success: true;
	faqs: Models.FAQ[];
};
export type FailedFetchFAQsState = {
	isLoading: false;
	success: false;
	error: Errors.GenericError;
};
export type FetchFAQsResponse = SuccessfulFetchFAQsState | FailedFetchFAQsState;
export type FetchFAQsState = InitialFetchFAQsState | LoadingFetchFAQsState | FetchFAQsResponse;

export type UseFetchFAQsInput = {
	autoFetch: boolean;
};
export type UseFetchFAQsOutput = [FetchFAQsState, () => Promise<FetchFAQsResponse>];
