import { useCallback, useContext, useEffect, useRef, useState } from "react";
import * as API from "../api";
import { RootStoreContext } from "../context/RootStoreContext";
import * as Errors from "../errors";
import * as Models from "../models";

export const useFetchLatestCourseClasses = ({
	autoFetch
}: UseFetchLatestCourseClassesInput): UseFetchLatestCourseClassesOutput => {
	const rootStore = useContext(RootStoreContext);
	const { courseClassStore, userStore } = rootStore;
	const requestRef = useRef<API.FetchLatestCourseClassesRequest>();
	const [state, setState] = useState<FetchLatestCourseClassesState>({ isLoading: false });

	const fetch = useCallback<() => Promise<FetchLatestCourseClassesResponse>>(async () => {
		setState({ isLoading: true });

		const request: API.FetchLatestCourseClassesRequest = { authToken: userStore.authToken };

		requestRef.current = request;
		const response = await API.fetchLatestCourseClasses(request);

		const newState: FetchLatestCourseClassesResponse = response.success
			? {
					isLoading: false,
					success: true,
					courseClasses: response.courseClasses
			  }
			: {
					isLoading: false,
					success: false,
					error: response.error
			  };

		if (requestRef.current !== request) return newState;

		if (response.success) {
			response.courseClasses.forEach(courseClass => courseClassStore.saveCourseClass(courseClass));
			courseClassStore.latestCourseClasses = response.courseClasses;
		}

		setState(newState);
		return newState;
	}, [courseClassStore]);

	useEffect(() => {
		setState({ isLoading: false });
		if (autoFetch && courseClassStore.latestCourseClasses.length === 0) fetch();

		return () => (requestRef.current = undefined);
	}, [autoFetch, fetch, courseClassStore]);

	return [state, fetch];
};

export type InitialFetchLatestCourseClassesState = {
	isLoading: false;
	success?: undefined;
};
export type LoadingFetchLatestCourseClassesState = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulFetchLatestCourseClassesState = {
	isLoading: false;
	success: true;
	courseClasses: Models.CourseClass[];
};
export type FailedFetchLatestCourseClassesState = {
	isLoading: false;
	success: false;
	error: Errors.GenericError;
};
export type FetchLatestCourseClassesResponse =
	| SuccessfulFetchLatestCourseClassesState
	| FailedFetchLatestCourseClassesState;
export type FetchLatestCourseClassesState =
	| InitialFetchLatestCourseClassesState
	| LoadingFetchLatestCourseClassesState
	| FetchLatestCourseClassesResponse;

export type UseFetchLatestCourseClassesInput = {
	autoFetch: boolean;
};
export type UseFetchLatestCourseClassesOutput = [
	FetchLatestCourseClassesState,
	() => Promise<FetchLatestCourseClassesResponse>
];
