import { action, observable } from "mobx";
import * as API from "../api";
import * as Errors from "../errors";
import * as Models from "../models";
import { Base } from "./Base";

export class CourseStore extends Base {
	@observable public courseList: Models.Course[] = [];

	@observable public courses = new Map<number, Models.Course>();
	@observable public coursesByCode = new Map<string, Models.Course>();

	@observable public fetchCourseByCodeStates = new Map<string, FetchCourseByCodeState>();

	@action public saveCourse(course: Models.Course): Models.Course {
		const { courses, coursesByCode } = this;
		const { courseClassListStore, userStore } = this.rootStore;

		let savedCourse = courses.get(course.id);

		if (savedCourse === course) return course;

		if (savedCourse === undefined) {
			courses.set(course.id, course);
			savedCourse = courses.get(course.id)!;
		}

		// Reconciliation
		if (course.classLists)
			for (const cl of course.classLists) {
				cl.course = undefined;
				courseClassListStore.saveCourseClassList(cl);
				cl.course = savedCourse;
			}

		if (course.createdBy) course.createdBy = userStore.saveUser(course.createdBy);
		if (course.updatedBy) course.updatedBy = userStore.saveUser(course.updatedBy);
		if (course.deletedBy) course.deletedBy = userStore.saveUser(course.deletedBy);

		for (const key of Object.keys(course)) {
			const value = course[key];

			if (value !== undefined) savedCourse![key] = value;
		}

		if (savedCourse.code !== undefined) coursesByCode.set(savedCourse.code, savedCourse);

		return savedCourse;
	}

	@action public deleteCourse(course: Models.Course | number | string): boolean {
		const { courses, coursesByCode } = this;

		let courseId: number | undefined =
			typeof course === "number" ? course : typeof course === "object" ? course.id : undefined;

		if (typeof course === "string") {
			const savedCourseId = coursesByCode.get(course);

			if (savedCourseId) courseId = savedCourseId.id;
		}

		const savedCourse = courseId === undefined ? undefined : courses.get(courseId);

		if (!savedCourse) return false;

		courses.delete(savedCourse.id);
		coursesByCode.delete(savedCourse.code);

		return true;
	}

	public async fetchCourseByCode(request: FetchCourseByCodeRequest): Promise<FetchCourseByCodeResponse> {
		const { fetchCourseByCodeStates } = this;
		const { courseCode } = request;

		const setState = <T extends FetchCourseByCodeState>(state: T): T => {
			fetchCourseByCodeStates.set(courseCode, state);
			return fetchCourseByCodeStates.get(courseCode) as T;
		};

		setState({ isLoading: true });
		const response = await API.fetchCourseByCode({ code: courseCode });

		if (response.success) {
			const course = this.saveCourse(response.course);

			return setState({
				isLoading: false,
				success: true,
				course
			});
		}

		return setState({
			isLoading: false,
			success: false,
			error: response.error
		});
	}
}

export type FetchCourseByCodeRequest = {
	courseCode: string;
};
export type InitialFetchCourseByCodeState = {
	isLoading: false;
	success?: undefined;
};
export type LoadingFetchCourseByCodeState = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulFetchCourseByCodeState = {
	isLoading: false;
	success: true;
	course: Models.Course;
};
export type FailedFetchCourseByCodeState = {
	isLoading: false;
	success: false;
	error: Errors.GenericError;
};
export type FetchCourseByCodeResponse = SuccessfulFetchCourseByCodeState | FailedFetchCourseByCodeState;
export type FetchCourseByCodeState =
	| InitialFetchCourseByCodeState
	| LoadingFetchCourseByCodeState
	| FetchCourseByCodeResponse;
